AddCSLuaFile()

ENT.Base = "beds_base"
ENT.PrintName = "Custom Bed"
ENT.Category = "Beds"
ENT.Author = "8Z"
ENT.Spawnable = true
ENT.AdminOnly = GetConVar("beds_custom_adminonly"):GetBool() --true

ENT.Model = "models/hunter/plates/plate1x2.mdl"
ENT.AngleOffset = Angle(0,0,0)
ENT.SpawnDelay = 60
ENT.BedSpawnOffset = Vector(0,0,8)
ENT.MaxHealth = 100
ENT.Mass = 100

ENT.Editable = true

DEFINE_BASECLASS(ENT.Base)

function ENT:SetupDataTables()
	self:NetworkVar("Entity", 0, "BedOwner")
	
	-- Appearance
	self:NetworkVar("Int", 0, 		"BedHealth", 	{KeyName = "BedHealth",
			Edit = {category = "Appearance", type = "Int", order = 1, min = 0, max = 500, }})
	self:NetworkVar("String", 0,	"BedModel", 	{KeyName = "BedModel",	
			Edit = {category = "Appearance", type = "Generic", 	order = 2, waitforenter = true}})
	self:NetworkVar("Int", 1, 		"BedMass", 	{KeyName = "BedMass", 
			Edit = {category = "Appearance", type = "Int", order = 3, min = 10, max = 500, }})
	self:NetworkVar("Bool", 1, 		"BedDraw", 	{KeyName = "BedDraw",
			Edit = {category = "Appearance", type = "Boolean", 	order = 4}})
			
	-- Attributes
	self:NetworkVar("String", 1, 	"BedName", 	{KeyName = "BedName",
			Edit = {category = "Attributes", type = "Generic", 	order = 11, waitforenter = true}})
	self:NetworkVar("Float", 0, 	"SpawnDelay", 	{KeyName = "SpawnDelay",
			Edit = {category = "Attributes", type = "Int", 		order = 12, min = 0, max = 600, }})
	self:NetworkVar("Bool", 0, 		"NeedIndoors", 	{KeyName = "NeedIndoors",
			Edit = {category = "Attributes", type = "Boolean", 	order = 13}})
end


if SERVER then

	function ENT:SpawnFunction( ply, tr, ClassName )

		local ent = BaseClass.SpawnFunction( self, ply, tr, ClassName )
		
		if IsValid(ent) then
			timer.Simple(0.5, function() if IsValid(ply) then
				BedMessage(ply, "Right click and use 'Edit Properties' to change properties of this bed!", 3, nil, "custom_spawn")
			end end) --
			timer.Simple(2.5, function() if IsValid(ply) then
				BedMessage(ply, "By default, only admins can spawn and edit this. Change this in the settings if you want.", 3, nil, "custom_spawnadmin")
			end end)
		end
		
		return ent
	end
	
	function ENT:Initialize()
		BaseClass.Initialize(self)
		
		-- used by Edit Properties to only change values after some time has passed
		self.NewVariables = {}
		
		-- Initialize pretty values
		self:SetBedHealth(self.MaxHealth)
		self:SetBedModel(self.Model)
		self:SetBedMass(self.Mass)
		self:SetBedName(self.PrintName)
		self:SetSpawnDelay(self.SpawnDelay)
		self:SetNeedIndoors(self.NeedIndoors)
		self:SetBedDraw(true)
	end

	function ENT:Think()
	
		if self.UpdateTime ~= nil and self.UpdateTime + 2 < CurTime() then
		
			if IsValid(self.UpdatePlayer) then
				BedMessage(self.UpdatePlayer, "Your variables for the bed have been updated.")
			end
		
			if self.NewVariables["BedHealth"] then
				self.MaxHealth = self.NewVariables["BedHealth"]
				self:SetMaxHealth(self.NewVariables["BedHealth"])
				self:SetHealth(math.min(self:Health(), self:GetMaxHealth()))
				if IsValid(self.UpdatePlayer) and self.MaxHealth == 0 then
					BedMessage(self.UpdatePlayer, "You have set max health to 0 or lower, making the bed invincible.", 3, nil, "custom_invincible")
				end
			end
			if self.NewVariables["BedMass"] and self:GetPhysicsObject():IsValid() then
				self:GetPhysicsObject():SetMass(self.NewVariables["BedMass"])
			end
			if self.NewVariables["SpawnDelay"] then
				self.SpawnDelay = self.NewVariables["SpawnDelay"]
			end
			
			self.UpdateTime = nil
			self.NewVariables = {}
		end
	
		return BaseClass.Think(self)
	end
	
	function ENT:CalibrateBounds()
		local mins, maxs = self:GetPhysicsObject():GetAABB()
		
		self.BedSpawnOffset = {Vector(0, 0, maxs.z + 4)} -- We always want to spawn on top
		
		-- Left/Right
		if math.max(math.abs(maxs.x), math.abs(mins.x)) < 64 then
			table.insert(self.BedSpawnOffset, Vector(maxs.x + 16, 0, 0))
			table.insert(self.BedSpawnOffset, Vector(mins.x - 16, 0, 0))
		end
		
		-- Front/Back
		if math.max(math.abs(maxs.y), math.abs(mins.y)) < 64 then
			table.insert(self.BedSpawnOffset, Vector(0, maxs.y + 16, 0))
			table.insert(self.BedSpawnOffset, Vector(0, mins.y - 16, 0))
		end
		
		if IsValid(self.UpdatePlayer) then
			BedMessage(self.UpdatePlayer, "New spawn positions for the model (" .. #self.BedSpawnOffset .. " total) have been calibrated.")
		end	
	end

	hook.Add("VariableEdited", "Beds", function(ent, ply, key, val, editor)
	
		if ent:GetClass() ~= "beds_custom" then return end
		if not game.SinglePlayer() and not ply:IsAdmin() and GetConVar("beds_custom_adminonly"):GetBool() then
			BedMessage(ply, "Only admins can edit custom beds! Your changes won't have an effect!", 2, Color(255, 0, 0), "custom_adminonly")
		end
	
		--print(key, val)
		ent.UpdatePlayer = ply
		if key == "BedHealth" then
			ent.NewVariables[key] = math.max(math.Round(tonumber(val)), 0)
			ent.UpdateTime = CurTime()
		elseif key == "BedModel" then
			if util.IsValidModel(val) then
				local mins, maxs = ent:GetPhysicsObject():GetAABB()
				ent:SetModel(val)
				ent:PhysicsInit(SOLID_VPHYSICS)
				ent:SetMaterial(nil)
				if ent:GetPhysicsObject():IsValid() then
					ent:CalibrateBounds()
					local mins2, maxs2 = ent:GetPhysicsObject():GetAABB()
					ent:SetPos(ent:GetPos() + Vector(0, 0, mins.z - mins2.z))
				else
					ent.BedSpawnOffset = Vector(0,0,8)
					BedMessage(ply, "This model does not have physics! Are you using an effect or invalid model?", 2, Color(255,0,0))
				end
			else
				BedMessage(ply, "This model is invalid!", 2, Color(255,0,0))
				return true
			end
		elseif key == "BedMass" then
			ent.NewVariables["BedMass"] = math.max(math.Round(tonumber(val)), 10)
			ent.UpdateTime = CurTime()
		elseif key == "BedName" then
			ent.PrintName = val
		elseif key == "SpawnDelay" then
			ent.NewVariables["SpawnDelay"] = math.max(math.Round(tonumber(val), 2), 0)
			ent.UpdateTime = CurTime()
		elseif key == "NeedIndoors" then
			ent.NeedIndoors = tobool(val)
		elseif key == "BedDraw" then
			ent.DrawBed = tobool(val)
			ent:DrawShadow(tobool(val))
		else
			print("Unexpected key-value pair for custom bed: {" .. tostring(key) .. ", " .. tostring(val) .. "}")
			--ent[key] = val
		end
	end)
end

if CLIENT then

	function ENT:Initialize()
		BaseClass.Initialize(self)
		
		if GetConVar("beds_custom_adminonly"):GetBool() and not LocalPlayer():IsAdmin() then
			self.Editable = false -- Give the illusion that this isn't editable. Does this work?
		end
	end

	function ENT:GetOverlayText()

		local text = self:GetBedName() or self.PrintName
		if IsValid(self:GetBedOwner()) then
			text = self:GetBedOwner():GetName() .. "'s " .. text
		end

		return text
	end
	
end